using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text.RegularExpressions;

namespace Cyotek.Tools.AssemblyReferenceScanner
{
  internal class AssemblyScanner
  {
    #regionPrivateMemberDeclarations

    private AssemblyInfoCollection _results;

    #endregionPrivateMemberDeclarations

    #regionPublicConstructors

    public AssemblyScanner()
    {
      this.Exclusions = new List<string>();
    }

    #endregionPublicConstructors

    #regionPublicMethods

    public AssemblyInfoCollection Scan(string folder)
    {
      if (string.IsNullOrEmpty(folder))
        throw new ArgumentNullException("folder");
      else if (!Directory.Exists(folder))
        throw new DirectoryNotFoundException();

      _results = new AssemblyInfoCollection();

      this.ScanFiles(folder, "*.dll");
      this.ScanFiles(folder, "*.exe");

      return _results;
    }

    #endregionPublicMethods

    #regionPublicProperties

    public List<string> Exclusions { get; set; }

    #endregionPublicProperties

    #regionPrivateMethods

    private void ScanFile(string fileName)
    {
      Assembly assembly;

      try
      {
        assembly = Assembly.ReflectionOnlyLoadFrom(fileName);
      }
      catch (BadImageFormatException)
      {
        Debug.WriteLine(fileName);
        assembly = null;
      }

      if (assembly != null)
      {
        AssemblyInfo info;

        info = new AssemblyInfo()
        {
          FileName = fileName,
          Name = assembly.GetName().Name,
          Version = assembly.GetName().Version
        };

        foreach (AssemblyName dependency in assembly.GetReferencedAssemblies())
        {
          if (!this.ShouldSkip(dependency.Name))
          {
            info.Dependencies.Add(new AssemblyNameInfo()
            {
              Name = dependency.Name,
              Version = dependency.Version
            });
          }
        }

        _results.Add(info);
      }
    }

    private void ScanFiles(string folder, string mask)
    {
      foreach (string fileName in Directory.GetFiles(folder, mask, SearchOption.AllDirectories))
        this.ScanFile(fileName);
    }

    private bool ShouldSkip(string name)
    {
      bool result;

      result = false;
      foreach (string exclusion in this.Exclusions)
      {
        if (Regex.IsMatch(name, exclusion))
        {
          result = true;
          break;
        }
      }

      return result;
    }

    #endregionPrivateMethods
  }
}